//*-------------------------------------------------------------------------
//| Title:
//|    listdrag.c
//|
//| Purpose:
//|   LISTDRAG demonstrates how to drag a list box item from one list box to 
//|   another.     
//|                                                  
//| Development :  VC 10/30/92
//|
//| Written by Microsoft Product Support Services, Windows Developer Support
//|
//| CopyRight (c) 1992 Microsoft Corporation. All rights reserved.
//|
//*-------------------------------------------------------------------------

#include <windows.h>          
#include "listdrag.h"          

// Globals
HINSTANCE ghInst;               // Instance of application
HWND  ghwnd;                    // Toplevel window handle

// Character arrays for string table entries
char szMainWndClass[STR_LEN];  
char szPopupWndClass[STR_LEN];
char szTitle[STR_LEN];
char szMainMenu[STR_LEN];
char szMainIcon[STR_LEN];
char szDragCursor[STR_LEN];
char szNoDropCursor[STR_LEN];

HWND hwndListFrom, hwndListTo;            // List window source and destination
HWND hwndFrom, hwndTo;                    // Popup window source and destination
FARPROC lpListProc;                       // Original window procedure of Listbox
HCURSOR hCursorDrag;                      // Cursor to be used during drag
HCURSOR hCursorNoDrop;                    // Cursor to be used during drag 
                                          // to show that drop is not allowed

//***********************************************************************
// Function: WinMain
//
// Purpose: Called by Windows on app startup.  Initializes everything,
//          and enters a message loop.
//
// Parameters:
//    hInstance     == Handle to this instance.
//    hPrevInstance == Handle to last instance of app.
//    lpCmdLine     == Command Line passed into app.
//    nCmdShow      == How app should come up (i.e. minimized/normal)
//
// Returns: Return value from PostQuitMessage.
//
// Comments:
//
// History:  Date       Author          Reason
//           10/30/92   VC              Created
//****************************************************************************
int PASCAL WinMain (HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow)
{
   MSG msg;

   LoadString(hInstance, IDS_MAINWNDCLASS, szMainWndClass, STR_LEN);
   LoadString(hInstance, IDS_POPUPWNDCLASS, szPopupWndClass, STR_LEN);
   LoadString(hInstance, IDS_PROGNAME, szTitle, STR_LEN);
   LoadString(hInstance, IDS_MAINMENU, szMainMenu, STR_LEN);
   LoadString(hInstance, IDS_MAINICON, szMainIcon, STR_LEN);
   LoadString(hInstance, IDS_DRAGCURSOR, szDragCursor, STR_LEN);
   LoadString(hInstance, IDS_NODROPCURSOR, szNoDropCursor, STR_LEN);
      
   if (!hPrevInstance)
      if (!InitApplication(hInstance))
         return (FALSE);

   if (!InitInstance(hInstance, nCmdShow))
      return (FALSE);

   while (GetMessage(&msg, NULL, NULL, NULL))
   {
      TranslateMessage(&msg);
      DispatchMessage(&msg);
   }
   
   return (msg.wParam); // Returns the value from PostQuitMessage
}


//****************************************************************************
// Function: InitApplication
//
// Purpose: Called by WinMain on first instance of app.  Registers
//          the window class.
//
// Parameters:
//    hInstance == Handle to this instance.
//
// Returns: TRUE on success, FALSE otherwise.
//
// Comments:
//
// History:  Date       Author          Reason
//           10/30/92   VC              Created
//****************************************************************************

BOOL InitApplication (HINSTANCE hInstance)
{
   WNDCLASS wc;

   wc.style = CS_DBLCLKS;
   wc.lpfnWndProc = MainWndProc;
   wc.cbClsExtra = 0;
   wc.cbWndExtra = 0;
   wc.hInstance = hInstance;
   wc.hIcon = LoadIcon(hInstance, szMainIcon);
   wc.hCursor = LoadCursor(NULL, IDC_ARROW);
   wc.hbrBackground = COLOR_WINDOW + 1;
   wc.lpszMenuName = szMainMenu;
   wc.lpszClassName = szMainWndClass;
   
   if (RegisterClass(&wc))
   {
      wc.style = 0;
      wc.lpfnWndProc = PopupWndProc;             
      wc.cbClsExtra = 0;         
      wc.cbWndExtra = 0;       
      wc.hInstance = hInstance;      
      wc.hIcon = LoadIcon(NULL, IDI_APPLICATION);
      wc.hCursor = LoadCursor(NULL, IDC_ARROW);
      wc.hbrBackground = COLOR_WINDOW+1; 
      wc.lpszMenuName =  NULL;   
      wc.lpszClassName = szPopupWndClass; 
       
      return (RegisterClass(&wc));
   }
   else return FALSE;
}

//****************************************************************************
// Function: InitInstance
//
// Purpose: Called by WinMain on instance startup.  Creates and
//            displays the main, overlapped window.
//
// Parameters:
//    hInstance     == Handle to this instance.
//    nCmdShow      == How app should come up (i.e. minimized/normal)
//
// Returns: TRUE on success, FALSE otherwise.
//
// Comments:
//
// History:  Date       Author           Reason
//           10/30/92   VC               Created
//****************************************************************************

BOOL InitInstance (HINSTANCE hInstance, int nCmdShow)
{
  
   ghInst = hInstance;
   // Create Main Window
   ghwnd = CreateWindow(szMainWndClass, szTitle,
                       WS_OVERLAPPEDWINDOW,
                       CW_USEDEFAULT, CW_USEDEFAULT,
                       600, 400,
                       NULL, NULL, hInstance, NULL);
   if (!ghwnd)
      return FALSE;
   
   ShowWindow(ghwnd, nCmdShow);    // Show the window                        
   UpdateWindow(ghwnd);            // Sends WM_PAINT message
}

//****************************************************************************
// Function: MainWndProc
//
// Purpose: Message handler for main overlapped window.
//
// Parameters:
//    hwnd    == Handle to this window.
//    msg     == Message to process.
//    wParam  == Depends on message
//    lParam  == Depends on message
//
// Returns: Depends on message.
//
// Comments:
//
// History:  Date       Author              Reason
//           10/30/92   VC                  Created
//****************************************************************************
LRESULT CALLBACK MainWndProc (HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
   static FARPROC lpListSubClassProc;        // pointer to list box subclass procedure
   
   switch (msg)
   {     
      case WM_CREATE:
      {      
         char szDragSource[STR_LEN];
         char szDragDestination[STR_LEN];
         
         LoadString(ghInst, IDS_DRAGSOURCE, szDragSource, STR_LEN);
         LoadString(ghInst, IDS_DRAGDESTINATION, szDragDestination, STR_LEN);
         
         // Create source and destination popup windows and the listboxes contained in them
         hwndFrom = CreateWindow(szPopupWndClass, szDragSource,  
                             WS_POPUP | WS_VISIBLE | WS_CAPTION | WS_BORDER,            
                             50, 50, 250, 150, hwnd, NULL, ghInst, NULL);           
         hwndListFrom = CreateWindow("LISTBOX", "",  
                             WS_CHILD | WS_VISIBLE | WS_BORDER | WS_HSCROLL,            
                             10, 10, 200, 100, hwndFrom, NULL, ghInst, NULL);                             
         hwndTo = CreateWindow(szPopupWndClass, szDragDestination,
                             WS_POPUP | WS_VISIBLE | WS_CAPTION | WS_BORDER,            
                             350, 50, 250, 150, hwnd, NULL, ghInst, NULL);                             
         hwndListTo = CreateWindow("LISTBOX", "",  
                             WS_CHILD | WS_VISIBLE |  WS_BORDER | WS_HSCROLL,            
                             10, 10, 200, 100, hwndTo, NULL, ghInst,NULL);
           
         // Subclass the list box in the 'Drag Source' popup window. The 
         // subclass procedure contains the code that implements dragging
         lpListSubClassProc = MakeProcInstance((FARPROC)ListSubClassProc, ghInst);
         lpListProc = (FARPROC)SetWindowLong(hwndListFrom, GWL_WNDPROC, (LONG)lpListSubClassProc);
           
         // Add items to the list in the 'Drag Source' popup window
         SendMessage(hwndListFrom, LB_ADDSTRING, 0, (LONG)(LPSTR)"First");
         SendMessage(hwndListFrom, LB_ADDSTRING, 0, (LONG)(LPSTR)"Second");
         SendMessage(hwndListFrom, LB_ADDSTRING, 0, (LONG)(LPSTR)"Third");
         SendMessage(hwndListFrom, LB_ADDSTRING, 0, (LONG)(LPSTR)"Fourth");
         SendMessage(hwndListFrom, LB_ADDSTRING, 0, (LONG)(LPSTR)"Fifth");
         SendMessage(hwndListFrom, LB_ADDSTRING, 0, (LONG)(LPSTR)"Sixth");
         SendMessage(hwndListFrom, LB_ADDSTRING, 0, (LONG)(LPSTR)"Seventh");                   
           
         // Load cursors to be used during the drag
         hCursorDrag = LoadCursor(ghInst, szDragCursor);
         hCursorNoDrop = LoadCursor(ghInst, szNoDropCursor);
                                   
         break;
      }
      
      case WM_CLOSE:                
         // Restore the original list box procedure
         SetWindowLong(hwndListFrom, GWL_WNDPROC, (LONG) lpListProc);
         return DefWindowProc(hwnd, msg, wParam, lParam);
         break;
                             
      case WM_COMMAND:
         switch (wParam)
         {
            case IDM_ABOUT:               
            {
               DLGPROC lpDlgProc;
               char szAboutBoxName[STR_LEN];

               LoadString(ghInst, IDS_ABOUTBOXNAME, szAboutBoxName, STR_LEN);
               lpDlgProc = MakeProcInstance(AboutDlg, ghInst);
               DialogBox(ghInst, szAboutBoxName, hwnd, lpDlgProc);
               FreeProcInstance(lpDlgProc);
               break;
            }
            
            default:
               return 0L;
         }
         break;
         
      case WM_DESTROY:                
         PostQuitMessage(0);
         break;

      default:                         
         return DefWindowProc(hwnd, msg, wParam, lParam);
   }
   
   return NULL;
}

//****************************************************************************
// Function: PopupWndProc
//
// Purpose: Message handler for popup windows that contain the list.
//
// Parameters:
//    hwnd    == Handle to _this_ window.
//    msg == Message to process.
//    wParam  == Depends on message
//    lParam  == Depends on message
//
// Returns: Depends on message.
//
// Comments:
//
// History:  Date       Author              Reason
//           10/30/92    VC                  Created
//****************************************************************************

LRESULT CALLBACK PopupWndProc (HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
   switch (msg)
   {
      default:
         return (DefWindowProc(hwnd, msg, wParam, lParam));
   }
   return FALSE;         
}

//****************************************************************************
// Function: ListSubClassProc
//
// Purpose: Subclass procedure for the list box. Contains code to implement
//          dragging of a list box item from the subclassed list box into another list box.
//
// Parameters:
//    hwnd    == Handle to _this_ window.
//    message == Message to process.
//    wParam  == Depends on message
//    lParam  == Depends on message
//
// Returns: Depends on message.
//
// Comments:
//
// History:  Date       Author              Reason
//           10/30/92    VC                  Created
//****************************************************************************

LRESULT CALLBACK ListSubClassProc (HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
{
   static BOOL bTracking = FALSE;      // Indicated if we are tracking the mouse in the drag loop
   static BOOL bDrag = FALSE;          // Indicated if the drag has started (if the user has
                                       // moved the mouse outside the drag rectangle).
   static HCURSOR hCursorOld = NULL;
   
   switch (message)
   {
      case WM_CANCELMODE:
         // WM_CANCELMODE is sent to the window that has captured the mouse before
         // a message box or modal dialog is displayed. If we were dragging the item
         // cancel the drag.
         bTracking = FALSE;
         ReleaseCapture();
         if (bDrag)
            SetCursor(hCursorOld);
            
         break;
         
      case WM_LBUTTONDOWN:
      {
         BOOL bDragSuccess = FALSE;  // Was the list box item dragged into the destination?        
         MSG msg;
         char szDragItem[30];        // Text of item being dragged
         int nDragItem,              // Index of item that is being dragged
             nCXFrame, nCYFrame;     // Dimensions of a thick frame. Used to construct 
                                     //  a rectangle to determine if the user started
                                     //  dragging an item or merely clicked on it.  
         RECT rectIsDrag;            // Rectangle to determine if dragging has started.                       
         POINT pt;
         HWND hwndWFP;               // Window handle returned by WindowFromPoint().
              

         // Pass the WM_LBUTTONDOWN to the list box window procedure. Then
         // fake a WM_LBUTTONUP so that we can track the drag.
         CallWindowProc(lpListProc, hwnd, message, wParam, lParam);
         CallWindowProc(lpListProc, hwnd, WM_LBUTTONUP, wParam, lParam);
         

         // Create a tiny rectangle to determine if item was dragged or merely clicked on.
         // If the user moves outside this rectangle we assume that the dragging has
         // started.
         pt.x = LOWORD(lParam);
         pt.y = HIWORD(lParam);                  
         nCXFrame = GetSystemMetrics(SM_CXFRAME);
         nCYFrame = GetSystemMetrics(SM_CYFRAME);
         SetRect(&rectIsDrag,  pt.x - (nCXFrame<<1), pt.y - (nCYFrame<<1),
                               pt.x + (nCXFrame<<1), pt.y + (nCYFrame<<1));
                               
         bTracking = TRUE;       // Track the drag until bTracking is set to FALSE
         
         // Capture the mouse and determine which item was selected in the source list                      
         SetCapture(hwnd);
         nDragItem = (int)SendMessage(hwndListFrom, LB_GETCURSEL, 0, 0);
         SendMessage(hwndListFrom, LB_GETTEXT, nDragItem, (LPARAM)(LPSTR)szDragItem);
         
         // Drag loop                      
         while (bTracking)
         {  
            // Retrieve mouse messages and keyboard messages. We retrieve keyboard
            // messages so that the system queue is not filled by keyboard messages
            // during the drag (This can happen if the user madly types while dragging!)
            // If neither mouse nor keyboard messages are available we wait. Both
            // PeekMessage() and Waitmessage() will yield to other apps.                             
            while (!PeekMessage(&msg, NULL, WM_MOUSEFIRST, WM_MOUSELAST, PM_REMOVE)
                   && !PeekMessage(&msg, NULL, WM_KEYFIRST, WM_KEYLAST, PM_REMOVE))
               WaitMessage();      
           
            switch(msg.message)
            {
               case WM_MOUSEMOVE:
                  if (!bDrag)
                  {
                     // Check if the user has moved out of the Drag rect. This indicates
                     // that the drag has started.
                     if (!PtInRect(&rectIsDrag, pt))
                     {
                        hCursorOld = SetCursor(hCursorDrag);
                        bDrag = TRUE;     // Drag has started
                     }
                  }
                  
                  pt.x = LOWORD(msg.lParam);
                  pt.y = HIWORD(msg.lParam);
                  
                  if (bDrag)
                  {  
                     // Change the cursor depending on where the mouse is to provide
                     // a visual indication of where the item is to be dropped.
                     ClientToScreen(hwnd, &pt);
                     hwndWFP = WindowFromPoint(pt);
                     if (hwndWFP == hwndFrom || hwndWFP == hwndListFrom || 
                           hwndWFP == hwndTo || hwndWFP == hwndListTo)     
                        SetCursor(hCursorDrag);
                     else SetCursor(hCursorNoDrop);
                  }
                  break;
                  
               
               case WM_LBUTTONUP: 
                  // End of Drag                             
                  pt.x = LOWORD(msg.lParam);
                  pt.y = HIWORD(msg.lParam);
                  
                  bTracking = FALSE;
                  ClientToScreen(hwnd, &pt);
                  hwndWFP = WindowFromPoint(pt);
                  if (hwndWFP == hwndTo || hwndWFP == hwndListTo)                 
                    bDragSuccess = TRUE;
                  break;
             
               default:
                  // Process the keyboard messages
                  TranslateMessage(&msg);
                  DispatchMessage(&msg);
            }
         }

         ReleaseCapture();
         if (bDrag)
         {
            SetCursor(hCursorOld);
            // Add item that was dragged to the destination list
            if (bDragSuccess)
               SendMessage(hwndListTo, LB_ADDSTRING, 0, (LONG)(LPSTR)szDragItem);
            bDrag = FALSE;
         }

         break; 
      }
         
      default:
         return CallWindowProc(lpListProc, hwnd, message, wParam, lParam);
   }
   return FALSE;         
}


//****************************************************************************
// Function: AboutDlg
//
// Purpose:  About dialog box function.
//
// Parameters:
//    hwndDlg    == Handle to this window.
//    msg        == Message to process.
//    wParam     == Depends on message
//    lParam     == Depends on message
//
// Returns: Depends on message.
//
// Comments:
//
// History:  Date       Author        Reason
//           1/27/92                  Created
//****************************************************************************

BOOL CALLBACK AboutDlg (HWND hwndDlg, UINT msg, WPARAM wParam, LPARAM lParam)
{
   switch (msg)
   {
      case WM_INITDIALOG:
         return TRUE;

      case WM_COMMAND:
         if ((wParam == IDOK) ||       // "OK" box selected?        
             (wParam == IDCANCEL))     // System menu close command?
         {
            EndDialog(hwndDlg, TRUE);     // Exits the dialog box 
            return TRUE;
         }
         break;
   }
   return FALSE;
}

